#include <qdir.h>
#include <qfile.h>

#include <qhttpservletrequest.h>
#include <qhttpservletresponse.h>
#include <qservletconfig.h>

#include "docwriter.h"
#include "fileservlet.h"

FileServlet::FileServlet( QServletConfig *cfg )
   : QHttpServlet( cfg, "File Servlet" )
{
}

FileServlet::~FileServlet()
{
}

bool FileServlet::serveFile( const QString &path, QHttpServletResponse *res )
{
   QFile file( path );
   if ( !file.open( IO_ReadOnly ) )
      return res->sendError( QHttpServletResponse::StatusInternalServerError );

   *(res->outputStream()) << QString(file.readAll());
   res->setStatus( QHttpServletResponse::StatusOk );
   
   return true;
}

bool FileServlet::serveDirectory( const QString &path, QHttpServletResponse *res )
{
   QString doc;
   QStringList files( QDir(path).entryList() );
   for ( QStringList::Iterator it = files.begin() ; it != files.end() ; ++it )
      doc += QString("<li><a href=\"%1\">%2</a><br>\n").arg(*it).arg(*it);

   DocWriter dw( res->outputStream() );
   dw.startDoc( path );
   dw.para( QString("<ul>\n%1\n</ul>\n").arg(doc) );
   dw.endDoc();

   res->setStatus( QHttpServletResponse::StatusOk );

   return true;
}

bool FileServlet::doGet( QHttpServletRequest *req, QHttpServletResponse *res )
{
    // Build local path
    QString reqpath = servletConfig()->initParameter("DocumentRoot");
    reqpath += QDir::cleanDirPath( req->requestURI() );

    // Validate path
    QFileInfo info( reqpath );

    if ( !info.exists() )
      return res->sendError( QHttpServletResponse::StatusNotFound );
    if ( !info.isReadable() )
      return res->sendError( QHttpServletResponse::StatusForbidden );

    // Serve content
    if ( info.isFile() )
      return serveFile( info.filePath(), res );
    else if ( info.isDir() )
      return serveDirectory( info.filePath(), res );

    return res->sendMessage( "Yo yo yo" );
}

// Local Variables:
// c-basic-offset: 3
// End:

