#include <stdlib.h>

#include <qregexp.h>
#include <qsocket.h>
#include <qdir.h>
#include <qdom.h>
#include <qtextstream.h>

#include <qservletconfig.h>
#include <qservletcontext.h>
#include <qhttpservletrequest.h>
#include <qhttpservletrunner.h>

#include <echoservlet.h>
#include <fileservlet.h>

#include "qhttpserver.h"
#include "qhttpserver.moc"

QHttpServer::QHttpServer( int p )
    : QServerSocket( p, 0 ),
      port(p), name("qhttpserver")
{
   if ( !ok() ) {
      qWarning("Failed to bind to port %d", port);
      exit(1);
   }

   run = new QHttpServletRunner;
   setupServlets();
}

QHttpServer::~QHttpServer()
{
   delete run;
}

void QHttpServer::setupServlets()
{
   // Echo servlet
   QServletConfig *cfg = new QServletConfig( run->context() );
   QHttpServlet *sv = new EchoServlet( cfg, "Echo Servlet" );
   run->addServlet( "/echo", sv );

   // File servlet
   cfg = new QServletConfig( run->context() );
   cfg->setInitParameter( "DocumentRoot", QDir::currentDirPath() );

   QHttpServlet *sv2 = new FileServlet( cfg );
   run->addServlet( "/", sv2 );
}

bool QHttpServer::loadSettings( const QString &filename )
{
   // Load XML file
   QFile f( filename );
   if ( !f.open( IO_ReadOnly ) )
      return false;

   QDomDocument doc("httpsettings");
   if ( !doc.setContent( &f ) )
      return false;

   // Read settings XML

   return true;
}

void QHttpServer::newConnection( int socket )
{
    QSocket* s = new QSocket(this);
    connect(s,SIGNAL(readyRead()),this,SLOT(readClient()));
    connect(s,SIGNAL(delayedCloseFinished()),this,SLOT(discardClient()));
    s->setSocket(socket);
    emit newConnect();
}

void QHttpServer::readClient()
{
   QSocket *socket = (QSocket *) sender();
   if ( !socket->canReadLine() )
      return;

   // Create request
   QTextStream in( socket );      
   QHttpServletRequest *req = run->createRequest( &in );

   req->setProtocol( "http" );
   req->setServerName( name );
   req->setServerPort( port );
   req->setRemoteHost( socket->peerName() );
   req->setRemoteAddress( socket->peerAddress().toString() );

   // Create respose
   QHttpServletResponse *res = run->createResponse( socket, req );

   // process request
   run->processRequest( req, res );

   socket->close();
   emit wroteToClient();
}

void QHttpServer::discardClient()
{
    QSocket* socket = (QSocket*)sender();
    delete socket;
    emit endConnect();
}


