#include <assert.h>

#include "qhttpservletrequest.h"
#include "qhttpservletresponse.h"

#include "qhttpservlet.h"

QHttpServlet::QHttpServlet( QServletConfig *cfg, const char *inf )
   : QServlet(cfg, inf), d(0)
{
}

QHttpServlet::~QHttpServlet()
{
}

bool QHttpServlet::service( QServletRequest *areq, QServletResponse *ares )
{
   QHttpServletRequest *req = static_cast<QHttpServletRequest *>(areq);
   QHttpServletResponse *res = static_cast<QHttpServletResponse *>(ares);

   // Check if the clients cached page is still valid
   QDateTime ims = req->dateHeader("If-Modified-Since");
   if ( !ims.isNull() ) {
      QDateTime lm = lastModified( req );
      if ( !lm.isNull() && (lm < ims) ) {
         res->setStatus( QHttpServletResponse::StatusNotModified );
         return true;
      }      
   }

   // Despatch to handler method   
   bool ok;
   if ( req->method() == "GET" )
      ok = doGet( req, res );
   else if ( req->method() == "POST" )
      ok = doPost( req, res );
   else if ( req->method() == "OPTIONS" )
      ok = doOptions( req, res );
   else if ( req->method() == "DELETE" )
      ok = doDelete( req, res );
   else if ( req->method() == "PUT" )
      ok = doPut( req, res );
   else if ( req->method() == "TRACE" )
      ok = doTrace( req, res );
   else
      ok = res->sendError( QHttpServletResponse::StatusMethodNotAllowed, req->method() );

   return ok;
}

bool QHttpServlet::doGet( QHttpServletRequest */*req*/, QHttpServletResponse *res )
{
   return res->sendError( QHttpServletResponse::StatusNotImplemented );
}

bool QHttpServlet::doPost( QHttpServletRequest */*req*/, QHttpServletResponse *res )
{
   return res->sendError( QHttpServletResponse::StatusNotImplemented );
}

bool QHttpServlet::doOptions( QHttpServletRequest */*req*/, QHttpServletResponse *res )
{
   return res->sendError( QHttpServletResponse::StatusNotImplemented );
}

bool QHttpServlet::doPut( QHttpServletRequest */*req*/, QHttpServletResponse *res )
{
   return res->sendError( QHttpServletResponse::StatusNotImplemented );
}

bool QHttpServlet::doDelete( QHttpServletRequest */*req*/, QHttpServletResponse *res )
{
   return res->sendError( QHttpServletResponse::StatusNotImplemented );
}

bool QHttpServlet::doTrace( QHttpServletRequest */*req*/, QHttpServletResponse *res )
{
   return res->sendError( QHttpServletResponse::StatusNotImplemented );
}

QDateTime QHttpServlet::lastModified( QHttpServletRequest */*req*/ )
{
   return QDateTime();
}

// Local Variables:
// c-basic-offset: 3
// End:

