// -*- c++ -*-

#ifndef QHTTPSERVLET_H
#define QHTTPSERVLET_H

#include <qstring.h>
#include <qdatetime.h>
#include <qservlet.h>

class QHttpServletRequest;
class QHttpServletResponse;

/**
 * The base class of all HTTP servlets. This class provides a binding
 * between a host HTTP server and its servlets that provides sensibe
 * default implementations of the various methods. Implementing custom
 * servlets is easy because the defaults mean you only have to
 * reimplement the methods you actually use.
 *
 * @author Richard Moore, rich@kde.org
 */
class QHttpServlet : public QServlet
{
public:
	QHttpServlet( QServletConfig *cfg, const char *info=0 );
   virtual ~QHttpServlet();

   /**
    * Reimplemented to call the appropriate doXXX(...) method. Returns
    * true if the request was successful, and false otherwise. Generally,
    * subclasses should not reimplement this method, but should instead
    * use the doXXX(...) methods.
    */
   virtual bool service( QServletRequest *req, QServletResponse *res );

   /**
    * Returns the time the results of the specified request last changed.
    * The default implementation returns a null time to indicate that the
    * content should always be reloaded.
    */
   virtual QDateTime lastModified( QHttpServletRequest *req );

protected:
   /** Called when service(...) receives a GET request. */
   virtual bool doGet( QHttpServletRequest *req, QHttpServletResponse *res );

   /** Called when service(...) receives a POST request. */
   virtual bool doPost( QHttpServletRequest *req, QHttpServletResponse *res );

   /** Called when service(...) receives a OPTIONS request. */
   virtual bool doOptions( QHttpServletRequest *req, QHttpServletResponse *res );
   
   /** Called when service(...) receives a PUT request. */
   virtual bool doPut( QHttpServletRequest *req, QHttpServletResponse *res );

   /** Called when service(...) receives a DELETE request. */
   virtual bool doDelete( QHttpServletRequest *req, QHttpServletResponse *res );

   /** Called when service(...) receives a TRACE request. */
   virtual bool doTrace( QHttpServletRequest *req, QHttpServletResponse *res );

private:
   class QHttpServletPrivate *d;
};

#endif // QHTTPSERVLET_H

// Local Variables:
// c-basic-offset: 3
// End:

