// -*- c++ -*-

#ifndef QHTTPSERVLETREQUEST_H
#define QHTTPSERVLETREQUEST_H

#include <qstring.h>
#include <qdatetime.h>
#include <qmap.h>
#include <qurl.h>

#include <qservletrequest.h>

/**
 * An HTTP servlet request.
 *
 * @author Richard Moore, rich@kde.org
 */
class QHttpServletRequest : public QServletRequest
{
public:
   QHttpServletRequest();
   virtual ~QHttpServletRequest();

   /** Returns the method of the request. */
   QString method() const { return meth; }

   /** Returns the path the client requested. */
   QString pathInfo() const { return url.path(); }

   /** Returns the query section of the request. */
   QString queryString() const { return url.query(); }

   /** Returns the URI the client requested. */
   QString requestURI() const { return uri; }

   /** Returns the HTTP version used by the client, for example '1.0' or '0.9'. */
   QString version() const { return ver; }
   
   //
   // Accessors for HTTP Headers.
   //

   /** Returns the names of the HTTP headers. */
   QStringList headerNames() const { return hdrNames; }

   /** Returns the value of the named header. */
   QString header( const QString &name ) const { return headers[name]; }

   /**
    * Returns the value of the named header as a QDateTime. If the header
    * did not form a value timestamp, you may find this method returns a
    * null QDateTime.
    */
   QDateTime dateHeader( const QString &name ) const;

   /**
    * Returns the int value of the named header or 0 if it could not be parsed. The
    * ok parameter will be true if the value was parsed ok, false otherwise.
    */
   int intHeader( const QString &name, bool *ok=0 ) const;

   //
   // Request parsing methods
   //

   bool readRequest( QTextStream *in );
   bool readMethod( const QString &line );
   bool hasHeaders() const { return hasHdrs; }
   bool readHeader( const QString &line );
   bool readHeaders( const QStringList &lines );

   /** Returns a string describing the contents of the request. */
   QString dump();

protected:
   void addHeader( const QString &name, const QString &val );
   
private:
   QString meth;
   QString uri;
   QUrl url;
   QString ver;
   QStringList hdrNames;
   QMap<QString,QString> headers; // name -> value

   bool hasHdrs;
};

#endif // QHTTPSERVLETREQUEST_H

// Local Variables:
// c-basic-offset: 3
// End:

