#include <qdatetime.h>

#include "docwriter.h"

#include "qhttpservletresponse.h"

QHttpServletResponse::QHttpServletResponse( QIODevice *out, QHttpServletRequest *req )
   : QServletResponse(out, req), stat(StatusNull)
{
}

QHttpServletResponse::~QHttpServletResponse()
{
}

void QHttpServletResponse::setHeader( const QString &name, const QString &value )
{
   headers[name] = value;
}

void QHttpServletResponse::setHeader( const QString &name, int value )
{
   headers[name] = QString("%1").arg(value);
}

void QHttpServletResponse::setHeader( const QString &name, const QDateTime &value )
{
   headers[name] = value.toString();
}

bool QHttpServletResponse::sendMessage( const QString &msg )
{
   if ( stat == StatusNull )
      stat = StatusOk;

   DocWriter dw( outputStream() );
   dw.startDoc( msg );
   dw.startSection( msg );
   dw.endSection();
   dw.endDoc();

   return true;
}

bool QHttpServletResponse::sendError( int status, const QString &msg )
{
	setStatus( status );
	return sendMessage( msg );
}

bool QHttpServletResponse::sendRedirect( const QString &location )
{
   setHeader( "Location", location );
   setStatus( StatusMovedPermanently );
	return true;
}

QTextStream *QHttpServletResponse::outputStream()
{
   if ( !hasOutputStream() ) {
      QTextStream *ts = QServletResponse::outputStream();
      *ts << QString("HTTP/%1 %2 %3")
             .arg(request()->version())
             .arg(stat)
             .arg(statusString(stat))
          << endl;
      sendHeaders( ts );
   }
      
   return QServletResponse::outputStream();
}

void QHttpServletResponse::sendHeaders( QTextStream *ts )
{
   QMapIterator<QString, QString> it;
   for ( it = headers.begin() ; it != headers.end() ; ++it )
      *ts << it.key() << ": " << it.data() << endl;
   *ts << endl;
}

QString QHttpServletResponse::statusString( int status )
{
   return QString("OK");
}

// Local Variables:
// c-basic-offset: 3
// End:
