// -*- c++ -*-

#ifndef QHTTPSERVLETRESPONSE_H
#define QHTTPSERVLETRESPONSE_H

#include <qmap.h>
#include <qstring.h>

#include <qhttpservletrequest.h>
#include <qservletresponse.h>

class QDateTime;
class QHttpServletRequest;

/**
 * An HTTP servlet response.
 */
class QHttpServletResponse : public QServletResponse
{
public:
   /** Standard HTTP status codes. */
   enum ResponseStatus {
      StatusNull = 0,
      
      StatusContinue = 100,
      StatusSwitchingProtocols = 101,
      
      StatusOk = 200,
      StatusCreated = 201,
      StatusAccepted = 202,
      StatusNonAuthoritativeInformation = 203,
      StatusNoContent = 204,
      StatusResetContent = 205,
      StatusPartialContent = 206,
      
      StatusMultipleChoices = 300,
      StatusMovedPermanently = 301,
      StatusMovedTemporarily = 302,
      StatusSeeOther = 303,
      StatusNotModified = 304,
      StatusUseProxy = 305,

      StatusBadRequest = 400,
      StatusUnauthorised = 401,
      StatusPaymentRequired = 402,
      StatusForbidden = 403,
      StatusNotFound = 404,
      StatusMethodNotAllowed = 405,
      // TODO: 406 - 415
      
      StatusInternalServerError = 500,
      StatusNotImplemented = 501
      // TODO: 502 - 505
   };

   QHttpServletResponse( QIODevice *out, QHttpServletRequest *req );
   virtual ~QHttpServletResponse();

   /**
    * Returns a text stream open for writing the output. Note that the
    * HTTP headers will be send the first time this method is called.
    */
   virtual QTextStream *outputStream();

   /** Sets the response HTTP status code. */
   void setStatus( int status ) { stat = status; }

   /** Returns the response HTTP status code. */
   int status() const { return stat; }

   virtual QHttpServletRequest *request() const {
      return (QHttpServletRequest *) QServletResponse::request();
   }
   
   //
   // Convenience methods for sending standard responses.
   //

   /** Sends a page that displays a message to the client. */
   virtual bool sendMessage( const QString &msg=QString::null );

   /** Sends an HTTP error to the client, with the specified status and message. */
   virtual bool sendError( int status, const QString &msg=QString::null );

   /** Sends an HTTP redirect to the client. */
   virtual bool sendRedirect( const QString &location );

   //
   // HTTP header methods.
   //

   //@{
   /** Sets the value of the named HTTP response header. */
   void setHeader( const QString &name, const QString &value );
   void setHeader( const QString &name, int value );
   void setHeader( const QString &name, const QDateTime &value );
   //@}

   /** Writes the HTTP headers to the specified text stream. */
   void sendHeaders( QTextStream *out );

   static QString statusString( int status );
private:
   int stat;
   QMap<QString,QString> headers; // name -> value
};

#endif // QHTTPSERVLETRESPONSE_H

// Local Variables:
// c-basic-offset: 3
// End:

