#include <assert.h>
#include <stdlib.h>

#include <qregexp.h>
#include <qiodevice.h>

#include <qservletbase.h>
#include <qservletconfig.h>
#include <qservletcontext.h>

#include "qhttpservlet.h"
#include "qhttpservletrequest.h"
#include "qhttpservletresponse.h"

#include "qhttpservletrunner.h"

ulong QHttpServletRunner::session_number = 0L;

QHttpServletRunner::QHttpServletRunner()
   : banner("QHttpServletRunner/0.1"), d(0)
{
  ctx = createContext();
}

QHttpServletRunner::~QHttpServletRunner()
{
   QValueListIterator<QCString> it;
   for ( it = paths.begin() ; it != paths.end() ; ++it )
      delete srvs[*it];
}

void QHttpServletRunner::addServlet( const char *path, QHttpServlet *sv )
{
   srvs.insert( path, sv );
   paths += path;
}

bool QHttpServletRunner::removeServlet( const char *path )
{
   bool removed = srvs.remove( path );
   if ( !removed )
      return false;

   paths.remove( path );
   return true;
}

QServletContext *QHttpServletRunner::createContext()
{
    return new QServletContext( "Command line test server" );
}

QHttpServletRequest *QHttpServletRunner::createRequest( QTextStream *in )
{
   assert( in && (!in->atEnd()) );

   QHttpServletRequest *req = new QHttpServletRequest();
   req->readRequest( in );
   return req;
}

QHttpServletResponse *QHttpServletRunner::createResponse( QIODevice *out, QHttpServletRequest *req )
{
   QHttpServletResponse *res = new QHttpServletResponse( out, req );
   res->setHeader( "Server", banner );
   return res;
}

bool QHttpServletRunner::processRequest( QHttpServletRequest *req, QHttpServletResponse *res )
{
   QHttpServlet *sv = findServlet( req );
   if ( !sv ) {
      res->sendError( QHttpServletResponse::StatusNotFound );
      return false;
   }

	if ( !sv->service( req, res ) )
      return false;

   // Ensure the headers get sent even if there's no content
   if ( !res->hasOutputStream() )
      res->outputStream();
      
   return true;
}

QHttpServlet *QHttpServletRunner::findServlet( QHttpServletRequest *req )
{
  QValueListIterator<QCString> it;
  for ( it = paths.begin() ; it != paths.end() ; ++it ) {
      QRegExp re( QString("^%1/").arg(*it) );
      if ( (QString(*it) == req->requestURI()) || (req->requestURI().contains(re)) )
         return srvs[*it];
  }
  
  return 0;
};

QHttpSession *QHttpServletRunner::findSession( QHttpServletRequest *req )
{
   return 0;
}


QString QHttpServletRunner::createSessionId( QHttpServletRequest *req )
{
   QString raw("%1:%2:%3:%4:%5");
   raw = raw.arg( req->serverName() )
         .arg( req->serverPort() )
         .arg( req->remoteAddress() )
         .arg( session_number++ )
         .arg( rand() );

   return raw;
}
