// -*- c++ -*-

#ifndef QHTTPSERVLETRUNNER_H
#define QHTTPSERVLETRUNNER_H

#include <qservletbase.h>
#include <qasciidict.h>
#include <qcstring.h>
#include <qvaluelist.h>

class QTextStream;

class QHttpServlet;
class QHttpServletRequest;
class QHttpServletResponse;
class QHttpSession;

/**
 * This class manages the interface between the HTTP server and the servlets.
 * @author Richard Moore
 */
class QHttpServletRunner : public QServletBase
{
public: 
   QHttpServletRunner();
   virtual ~QHttpServletRunner();

   /** Returns the context the runner's servlets exist in. */
   QServletContext *context() { return ctx; }

   /**
    * Makes the specified servlet available at the specified path. Note
    * that the runner takes ownership of the servlet and will delete it
    * if the servlet is removed.
    */
   void addServlet( const char *path, QHttpServlet *sv );

   /**
    * Removes the servlet that is mapped at the specified path. Returns
    * false if no servlet was mapped at the path.
    */
   bool removeServlet( const char *path );
      
   /** Creates a request object. */
   QHttpServletRequest *createRequest( QTextStream *in );

   /** Creates a response object. */
   QHttpServletResponse *createResponse( QIODevice *out, QHttpServletRequest *req );

   /**
    * This method is called when the HTTP server asks us to handle a request.
    * It despatches the request to the appropriate servlet which will populate
    * the response object. If there is no servlet that can handle the request
    * then a 404 response will be generated. This method returns true if the
    * request was successfully handled, false otherwise.
    */
   bool processRequest( QHttpServletRequest *req, QHttpServletResponse *res );
   
   /** Returns the servlet that should handle the specified request or 0. */
   QHttpServlet *findServlet( QHttpServletRequest *req );

   /** Returns the session associated with the specified request or 0. */
   QHttpSession *findSession( QHttpServletRequest *req );
   
private:
   /** Creates the context the servlets exist in. */
   QServletContext *createContext();

   QString createSessionId( QHttpServletRequest *req );
private:
   QServletContext *ctx;
   QValueList<QCString> paths;
   QAsciiDict<QHttpServlet> srvs;
   QString banner;
   
   class QHttpServletRunnerData *d;
   static ulong session_number;
};

#endif

// Local Variables:
// c-basic-offset: 3
// End:
