// -*- c++ -*-

#ifndef QHTTPSESSION_H
#define QHTTPSESSION_H

#include <qdatetime.h>
#include <qmap.h>
#include <qstringlist.h>
#include <qvariant.h>

#include <qservletbase.h>

/**
 * Stores data about a session.
 *
 * @author Richard Moore
 */
class QHttpSession : public QServletBase
{
public: 
	QHttpSession();
	~QHttpSession();

   /** Invalidates the session prevent further use. */
   void invalidate();

   /** Returns true iff this is the first request for the session. */
   bool isNew() const { return isnew; }

   /** Specifies if this is a new session. */
   void setNew( bool isnew=false ) { this->isnew = isnew; }
  
   /** Returns the time at which session was created. */
   QDateTime creationTime() const { return created; }

   /** Returns the id of this session. */
   QString id() const { return sessid; }

   /**
    * Sets the time the user last accessed this session. If the time is null then
    * the current time is used.
    */
   void setLastAccessTime( const QDateTime &dt = QDateTime() );
   
   /** Returns the time the user last accessed this session. */
   QDateTime lastAccessedTime() const { return accessed; }

   /** Returns the interval after which this session becomes invalid if unused. */
   int maxInactiveInterval() const { return maxinactive; }

   /** Sets the interval after which this session becomes invalid if unused. */
   void setMaxInactiveInterval( int interval ) { maxinactive = interval; }

   //
   // Value handling
   //
   
   /** Returns the value bound to the specified name. */
   QVariant value( const QString &name ) const { return vals[name]; }

   /** Returns the names of all values bound to this session. */
   QStringList valueNames() const { return valnames; }

   /** Binds the specified value to the specified name. */
   void setValue( const QString &name, const QVariant &value );

   /** Removes the value bound to the specified name. */
   void removeValue( const QString &name ) { valnames.remove(name); vals.remove(name); }
      
private:
   QDateTime created;
   QDateTime accessed;
   QString sessid;
   int maxinactive;
   QStringList valnames;
   QMap<QString, QVariant> vals;
   bool isnew;

   class QHttpSessionData *d;
};

#endif
