// -*- c++ -*-

#ifndef QSERVLETREQUEST_H
#define QSERVLETREQUEST_H

#include <qmap.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qservletbase.h>

class QIODevice;
class QTextStream;

/**
 * Abstract base class of all servlet requests.
 *
 * @author Richard Moore, rich@kde.org
 */
class QServletRequest : public QServletBase
{
public:
   QServletRequest();
   virtual ~QServletRequest();

   //
   // Request content
   //
   
   /** Returns the character encoding the client used when forming the request. */
   QString characterEncoding() const { return enc; }
   void setCharacterEncoding( const QString &enc ) { this->enc = enc; }

   /** Returns the length of the content of the request. */
   long contentLength() const { return len; }
   void setContentLength( long len ) { this->len = len; }

   /** Returns the MIME type of the content of the request. */
   QString contentType() const { return mime; }
   void setContentType( const QString &mime ) { this->mime = mime; }

   /** Returns a QIODevice that is open for reading the request content. */
   virtual QIODevice *inputDevice() { return 0; }

   /** Returns a QTextStream that is open for reading the request content. */
   virtual QTextStream *inputStream() { return 0; }

   //
   // Request parameters
   //
   
   /** Returns the value of the named parameter. */
   QString parameter( const QString &name ) { return params[name]; }

   /** Returns a list of the parameter names. */
   QStringList parameterNames() { return paramNames; }

   //
   // Request info
   //
   
   /** Returns the protocol used to make the request. */
   QString protocol() { return proto; }
   void setProtocol( const QString &protocol ) { proto = protocol; }
   
   /** Returns the address the request was sent from. */
   QString remoteAddress() const { return remAddr; }
   
   /** Specifies the address the request was sent from. */
   void setRemoteAddress( const QString &addr ) { remAddr = addr; }

   /** Returns the hostname the request was sent from. */
   QString remoteHost() const { return remHost; }
   
   /** Specifies the address the request was sent from. */
   void setRemoteHost( const QString &host ) { remHost = host; }

   /** Returns the name of the server specified in the request. */
   QString serverName() const { return srvName; }
   
   /** Specifies the name of the server used in the request. */
   void setServerName( const QString &name ) { srvName = name; }

   /** Returns the port the request was made on. */
   int serverPort() const { return srvPort; }
   
   /** Specifies the port the request was made on. */
   void setServerPort( int port ) { srvPort = port; }

private:
   QString enc;
   long len;
   QString mime;

   QString proto;
   QString remAddr;
   QString remHost;
   QString srvName;
   int srvPort;

   QStringList paramNames;
   QMap<QString, QString> params;
};

#endif // QSERVLETREQUEST_H

// Local Variables:
// c-basic-offset: 3
// End:

