// -*- c++ -*-

#ifndef QSERVLETRESPONSE_H
#define QSERVLETRESPONSE_H

#include <qstring.h>
#include <qservletbase.h>

class QIODevice;
class QTextStream;
class QServletRequest;

/**
 * Abstract base class of all servlet responses.
 */
class QServletResponse : public QServletBase
{
public:
   QServletResponse( QIODevice *out, QServletRequest *req );
   virtual ~QServletResponse();

   /** Returns a QIODevice that is open for writing the response content. */
   QIODevice *outputDevice() { return dev; }

   virtual QServletRequest *request() const { return req; }
   
   /**
    * Returns a QTextStream that is open for writing the response content. The
    * stream is created when this method is first called.
    */
   virtual QTextStream *outputStream();

   /** Returns true iff the output stream has been created. */
   bool hasOutputStream() const { return out ? true : false; }
   
   /** Sets the length of the response content. */
   void setContentLength( long len ) { this->len = len; }

   /** Sets the MIME type of the response content. */
   void setContentType( const QString &mime ) { this->mime = mime; }

private:
   QIODevice *dev;
   QServletRequest *req;
   QTextStream *out;
   long len;
   QString mime;
};

#endif // QSERVLETRESPONSE_H

// Local Variables:
// c-basic-offset: 3
// End:

